@@ -208,7 +208,14 @@ clutter_stage_x11_get_geometry (ClutterStageWindow    *stage_window,
 
   geometry->x = geometry->y = 0;
 
-  /* If we're fullscreen, return the size of the display. */
+  /* If we're fullscreen, return the size of the display.
+   *
+   * FIXME - this is utterly broken for anything that is not a single
+   * head set up; the window manager will give us the right size in a
+   * ConfigureNotify, but between the fullscreen signal emission on the
+   * stage and the following frame, the size returned by the stage will
+   * be wrong.
+   */
   if (_clutter_stage_is_fullscreen (stage_cogl->wrapper) &&
       stage_x11->fullscreening)
     {
@@ -355,18 +362,21 @@ set_cursor_visible (ClutterStageX11 *stage_x11)
 
   if (stage_x11->is_cursor_visible)
     {
-#if 0 /* HAVE_XFIXES - seems buggy/unreliable */
-      XFixesShowCursor (backend_x11->xdpy, stage_x11->xwin);
+#if HAVE_XFIXES
+      if (stage_x11->cursor_hidden_xfixes)
+        {
+          XFixesShowCursor (backend_x11->xdpy, stage_x11->xwin);
+          stage_x11->cursor_hidden_xfixes = FALSE;
+        }
 #else
       XUndefineCursor (backend_x11->xdpy, stage_x11->xwin);
 #endif /* HAVE_XFIXES */
     }
   else
     {
-#if 0 /* HAVE_XFIXES - seems buggy/unreliable, check cursor in firefox 
-       *               loading page after hiding.  
-      */
+#if HAVE_XFIXES
       XFixesHideCursor (backend_x11->xdpy, stage_x11->xwin);
+      stage_x11->cursor_hidden_xfixes = TRUE;
 #else
       XColor col;
       Pixmap pix;
@@ -645,12 +655,16 @@ clutter_stage_x11_realize (ClutterStageWindow *stage_window)
    * the event mask we passed to XSelectInput as the template
    */
   device_manager = clutter_device_manager_get_default ();
-  _clutter_device_manager_select_stage_events (device_manager,
-                                               stage_cogl->wrapper,
-                                               event_flags);
+  if (G_UNLIKELY (device_manager != NULL))
+    {
+      _clutter_device_manager_select_stage_events (device_manager,
+                                                   stage_cogl->wrapper,
+                                                   event_flags);
 
-  g_signal_connect (device_manager, "device-added",
-                    G_CALLBACK (stage_events_device_added), stage_window);
+      g_signal_connect (device_manager, "device-added",
+                        G_CALLBACK (stage_events_device_added),
+                        stage_window);
+    }
 
   clutter_stage_x11_fix_window_size (stage_x11,
                                      stage_x11->xwin_width,
@@ -866,6 +880,7 @@ clutter_stage_x11_init (ClutterStageX11 *stage)
   stage->is_foreign_xwin = FALSE;
   stage->fullscreening = FALSE;
   stage->is_cursor_visible = TRUE;
+  stage->cursor_hidden_xfixes = FALSE;
   stage->accept_focus = TRUE;
 
   stage->title = NULL;
@@ -1143,6 +1158,26 @@ clutter_stage_x11_translate_event (ClutterEventTranslator *translator,
         }
       break;
 
+    case EnterNotify:
+#if HAVE_XFIXES
+      if (!stage_x11->is_cursor_visible && !stage_x11->cursor_hidden_xfixes)
+        {
+          XFixesHideCursor (backend_x11->xdpy, stage_x11->xwin);
+          stage_x11->cursor_hidden_xfixes = TRUE;
+        }
+#endif
+      break;
+
+    case LeaveNotify:
+#if HAVE_XFIXES
+      if (stage_x11->cursor_hidden_xfixes)
+        {
+          XFixesShowCursor (backend_x11->xdpy, stage_x11->xwin);
+          stage_x11->cursor_hidden_xfixes = FALSE;
+        }
+#endif
+      break;
+
     case Expose:
       {
         XExposeEvent *expose = (XExposeEvent *) xevent;
